@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2013-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
  *
  * @category  Horde
- * @copyright 2013 Horde LLC
+ * @copyright 2013-2014 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   Cache
  */
@@ -16,7 +16,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2013 Horde LLC
+ * @copyright 2013-2014 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   Cache
  * @since     2.2.0
@@ -44,13 +44,9 @@ class Horde_Cache_Storage_Hashtable extends Horde_Cache_Storage_Base
             throw new InvalidArgumentException('Missing hashtable parameter.');
         }
 
-        if (isset($params['prefix'])) {
-            $params['hashtable'] = clone $params['hashtable'];
-            $params['hashtable']->prefix = $params['prefix'];
-            unset($params['prefix']);
-        }
-
-        parent::__construct($params);
+        parent::__construct(array_merge(array(
+            'prefix' => ''
+        ), $params));
     }
 
     /**
@@ -61,35 +57,52 @@ class Horde_Cache_Storage_Hashtable extends Horde_Cache_Storage_Base
     }
 
     /**
-     * NOTE: This driver ignores the lifetime argument.
      */
     public function get($key, $lifetime = 0)
     {
-        return $this->_hash->get($key);
+        $dkey = $this->_getKey($key);
+        $query = array($dkey);
+        if ($lifetime) {
+            $query[] = $lkey = $this->_getKey($key, true);
+        }
+
+        $res = $this->_hash->get($query);
+
+        if ($lifetime &&
+            (!$res[$lkey] || (($lifetime + $res[$lkey]) < time()))) {
+            return false;
+        }
+
+        return $res[$dkey];
     }
 
     /**
      */
     public function set($key, $data, $lifetime = 0)
     {
-        $this->_hash->set($key, $data, array_filter(array(
-            'timeout' => $lifetime
-        )));
+        $opts = array_filter(array(
+            'expire' => $lifetime
+        ));
+
+        $this->_hash->set($this->_getKey($key), $data, $opts);
+        $this->_hash->set($this->_getKey($key, true), time(), $opts);
     }
 
     /**
-     * NOTE: This driver ignores the lifetime argument.
      */
     public function exists($key, $lifetime = 0)
     {
-        return $this->_hash->exists($key);
+        return ($this->get($key, $lifetime) !== false);
     }
 
     /**
      */
     public function expire($key)
     {
-        $this->_hash->delete($key);
+        $this->_hash->delete(array(
+            $this->_getKey($key),
+            $this->_getKey($key, true)
+        ));
     }
 
     /**
@@ -99,4 +112,17 @@ class Horde_Cache_Storage_Hashtable extends Horde_Cache_Storage_Base
         $this->_hash->clear();
     }
 
+    /**
+     * Return the hashtable key.
+     *
+     * @param string $key  Object ID.
+     * @param boolean $ts  Return the timestamp key?
+     *
+     * @return string  Hashtable key ID.
+     */
+    protected function _getKey($key, $ts = false)
+    {
+        return $this->_params['prefix'] . $key . ($ts ? '_t' : '');
+    }
+
 }
