@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2013-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
  *
  * @category  Horde
- * @copyright 2013 Horde LLC
+ * @copyright 2013-2014 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   Cache
  */
@@ -16,7 +16,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2013 Horde LLC
+ * @copyright 2013-2014 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   Cache
  */
@@ -39,8 +39,10 @@ class Horde_Cache_Storage_Mongo extends Horde_Cache_Storage_Base
      * Constructor.
      *
      * @param array $params  Parameters:
+     * <pre>
      *   - collection: (string) The collection name.
      *   - mongo_db: [REQUIRED] (Horde_Mongo_Client) A MongoDB client object.
+     * </pre>
      */
     public function __construct(array $params = array())
     {
@@ -59,7 +61,7 @@ class Horde_Cache_Storage_Mongo extends Horde_Cache_Storage_Base
     public function __destruct()
     {
         /* Only do garbage collection 0.1% of the time we create an object. */
-        if (rand(0, 999) == 0) {
+        if (substr(time(), -3) === '000') {
             try {
                 $this->_db->remove(array(
                     self::EXPIRE => array(
